@@ -1,8 +1,8 @@
 package Graph::Weighted;
 use strict;
 use Carp;
-use vars qw($VERSION); $VERSION = '0.08.1';
-use base qw(Graph::Directed);
+use vars qw($VERSION); $VERSION = '0.01';
+use base qw(Graph);
 
 use constant WEIGHT => 'weight';
 
@@ -18,7 +18,10 @@ sub new {  # {{{
 
     bless $self, $class;
 
-    $self->load($args{data}) if $args{data};
+    if ($args{data}) {
+        $self->load($args{data});
+        $self->graph_weight;
+    }
 
     return $self;
 }  # }}}
@@ -27,82 +30,53 @@ sub _debug {  # {{{
     print @_, "\n" if shift->{debug};
 }  # }}}
 
+sub reset_graph {  # {{{
+    my $self = shift;
+    undef $self->{data};
+    undef $self->{matrix};
+    undef $self->{heaviest_vertex};
+    undef $self->{lightest_vertex};
+    $self->delete_vertices($self->vertices);
+    $self->delete_attribute(WEIGHT);
+}  # }}}
+
 sub load {  # {{{
     my ($self, $data) = @_;
-$self->_debug('entering load with a '. ref ($data) . ' object');
+$self->_debug("entering load with $data");
 
-    if (ref ($data) eq 'Math::MatrixReal' ||
-        ref ($data) eq 'Math::Matrix'
-    ) {
-        # Math::Matrix* objects are LoLs and have the matrix as the
-        # first element.
-        $data = $data->[0];
-    }
-
-    if (ref ($data) eq 'Math::MatrixBool') {
-        # Math::MatrixBool objects are actually Bit::Vectors, that
-        # can be output by "stringification", like this:
-        # [ 1 0 0 ]
-        # [ 1 1 0 ]
-        # [ 1 1 1 ]
-        # So de-stringification gymnastics have to happen.  Ugh.
-        $data = [
-            map { [ split ] }
-                sprintf ('%s', "$data") =~ /\[\s([\d\s]+)\s\]/gs
-        ];
-    }
-
-    # We are given a HoH.
     if (ref ($data) eq 'HASH') {
-        # Set the vertices and weighted edges.
-        while (my ($vertex, $successors) = each %$data) {
-            # Add an "edgeless vertex" if there are no successors or
-            # successors all have zero values.
-            my $weight = 0;
-            $weight += $_ for values %$successors;
-$self->_debug("$vertex weight = $weight");
+        # Set the object matrix to the HoH data.
+        $self->{matrix} = $data;
 
-            unless (keys %$successors && ($weight || $self->{zero_edges})) {
+        # Set the vertices and weighted edges.
+        while (my ($vertex, $neighbors) = each %$data) {
+            unless (keys %$neighbors) {
 $self->_debug("$vertex edgeless");
                 $self->add_vertex($vertex);
             }
 
-            while (my ($successor, $n) = each %$successors) {
-$self->_debug("$vertex =($n)=> $successor");
-                $self->add_weighted_edge($vertex, $n, $successor)
+            while (my ($neighbor, $n) = each %$neighbors) {
+$self->_debug("$vertex =($n)=> $neighbor");
+                $self->add_weighted_edge($vertex, $n, $neighbor)
                     if $self->{zero_edges} || $n != 0;
             }
         }
     }
     # We are given a LoL.
     elsif (ref ($data) eq 'ARRAY') {
-        # Initialize the object data.
-        $self->{data} = {};
-
         # Set the vertices and weighted edges.
         for my $vertex (0 .. @$data - 1) {
             croak "Incorrectly sized array\n"
                 unless @{ $data->[$vertex] } == @$data;
 
-            my $weight = 0;
-            $weight += $_ for @{ $data->[$vertex] };
-$self->_debug("$vertex weight = $weight");
+            for my $neighbor (0 .. @{ $data->[$vertex] } - 1) {
+                my $n = $data->[$vertex][$neighbor];
 
-            unless ($weight || $self->{zero_edges}) {
-$self->_debug("$vertex edgeless");
-                $self->add_vertex($vertex);
-                $self->{data}{$vertex} = {};
-            }
+$self->_debug("$vertex =($n)=> $neighbor");
+                $self->add_weighted_edge($vertex, $n, $neighbor)
+                    if $self->{zero_edges} || $n != 0;
 
-            for my $successor (0 .. @{ $data->[$vertex] } - 1) {
-                my $n = $data->[$vertex][$successor];
-
-                if ($n || $self->{zero_edges}) {
-$self->_debug("$vertex =($n)=> $successor");
-                    $self->add_weighted_edge($vertex, $n, $successor);
-
-                    $self->{data}{$vertex}{$successor} = $n;
-                }
+                $self->{matrix}{$vertex}{$neighbor} = $n;
             }
         }
     }
@@ -112,8 +86,8 @@ $self->_debug("$vertex =($n)=> $successor");
 $self->_debug('exiting load');
 }  # }}}
 
-sub data {  # {{{
-    return shift->{data};
+sub matrix {  # {{{
+    return shift->{matrix};
 }  # }}}
 
 sub graph_weight {  # {{{
@@ -139,8 +113,6 @@ $self->_debug('exiting graph_weight');
 sub vertex_weight {  # {{{
     my ($self, $vertex, $weight) = @_;
 $self->_debug("entering vertex_weight with $vertex");
-    croak "Can't compute the weight of an undefined vertex."
-        unless defined $vertex;
 
     if (defined $weight) {
         # Distribute the weight to all outgoing edges.
@@ -149,17 +121,17 @@ $self->_debug("weight is defined as $weight");
         my $old = $self->get_attribute(WEIGHT, $vertex);
         $self->set_attribute(WEIGHT, $vertex, $weight);
 
-        # How many successors does the vertex have?
-        my $n_successors = keys %{ $self->{data}{$vertex} };
+        # How many neighbors does the vertex have?
+        my $n_neighbors = keys %{ $self->{matrix}{$vertex} };
 
         # Distribute the weight to all outgoing edges.
-        my $new_weight = $weight / $n_successors;
-        for my $successor (keys %{ $self->{data}{$vertex} }) {
-            # Reset the data value.
-            $self->{data}{$vertex}{$successor} = $new_weight;
+        my $new_weight = $weight / $n_neighbors;
+        for my $neighbor (keys %{ $self->{matrix}{$vertex} }) {
+            # Reset the matrix value.
+            $self->{matrix}{$vertex}{$neighbor} = $new_weight;
             # Reset the outgoing edge.
-            $self->set_attribute(WEIGHT, $vertex, $successor, $new_weight);
-$self->_debug("$vertex =($new_weight)=> $successor: new vertex weight set");
+            $self->set_attribute(WEIGHT, $vertex, $neighbor, $new_weight);
+$self->_debug("$vertex =($new_weight)=> $neighbor: new vertex weight set");
         }
 
         # Adjust the total graph weight if we made a change.
@@ -173,7 +145,7 @@ $self->_debug("adjust the graph weight to $graph_weight");
         }
     }
     elsif (!$self->has_attribute(WEIGHT, $vertex)) {
-        $weight += $_ for values %{ $self->{data}{$vertex} };
+        $weight += $_ for values %{ $self->{matrix}{$vertex} };
         $weight = 0 unless $weight;
 $self->_debug("weight computed as $weight");
         $self->set_attribute(WEIGHT, $vertex, $weight);
@@ -184,22 +156,20 @@ $self->_debug('exiting vertex_weight');
 }  # }}}
 
 sub edge_weight {  # {{{
-    my ($self, $vertex, $successor, $weight) = @_;
-$self->_debug("entering edge_weight with $vertex and $successor");
-    croak "Can't compute the weight of an edge bounded by undefined vertices."
-        unless defined $vertex && defined $successor;
+    my ($self, $vertex, $neighbor, $weight) = @_;
+$self->_debug("entering edge_weight with $vertex and $neighbor");
 
     if (defined $weight) {
 $self->_debug("weight is defined as $weight");
         # Out with the old; in with the new.
-        my $old = $self->get_attribute(WEIGHT, $vertex, $successor);
+        my $old = $self->get_attribute(WEIGHT, $vertex, $neighbor);
 
         # Reset the edge weight.
-        $self->set_attribute(WEIGHT, $vertex, $successor, $weight);
+        $self->set_attribute(WEIGHT, $vertex, $neighbor, $weight);
 
-        # Reset the data value.
-        $self->{data}{$vertex}{$successor} = $weight;
-$self->_debug("$vertex =($weight)=> $successor: new vertex weight set");
+        # Reset the matrix value.
+        $self->{matrix}{$vertex}{$neighbor} = $weight;
+$self->_debug("$vertex =($weight)=> $neighbor: new vertex weight set");
 
         # Adjust the graph and vertex weight if we made a change.
         if ($old != $weight) {
@@ -214,14 +184,14 @@ $self->_debug("adjust the vertex weight to $vertex_weight");
             $self->set_attribute(WEIGHT, $vertex, $vertex_weight);
         }
     }
-    elsif (!$self->has_attribute(WEIGHT, $vertex, $successor)) {
-        $weight = $self->{data}{$vertex}{$successor};
-$self->_debug("weight from the data is $weight");
-        $self->set_attribute(WEIGHT, $vertex, $successor, $weight);
+    elsif (!$self->has_attribute(WEIGHT, $vertex, $neighbor)) {
+        $weight = $self->{matrix}{$vertex}{$neighbor};
+$self->_debug("weight from the matrix is $weight");
+        $self->set_attribute(WEIGHT, $vertex, $neighbor, $weight);
     }
 
 $self->_debug('exiting edge_weight');
-    return $self->get_attribute(WEIGHT, $vertex, $successor);
+    return $self->get_attribute(WEIGHT, $vertex, $neighbor);
 }  # }}}
 
 sub heaviest_vertices {  # {{{
@@ -229,8 +199,6 @@ sub heaviest_vertices {  # {{{
 $self->_debug('entering heaviest_vertices');
 
     unless (defined $self->{heaviest_vertices}) {
-        $self->{heaviest_vertices} = [];
-
         my $heavy = 0;
 
         for ($self->vertices) {
@@ -256,8 +224,6 @@ sub lightest_vertices {  # {{{
 $self->_debug('entering lightest_vertices');
 
     unless (defined $self->{lightest_vertices}) {
-        $self->{lightest_vertices} = [];
-
         my $light;
 
         for ($self->vertices) {
@@ -278,20 +244,6 @@ $self->_debug('exiting lightest_vertices with ['. join (', ', @{ $self->{lightes
     return $self->{lightest_vertices};
 }  # }}}
 
-sub max_weight {  # {{{
-    my $self = shift;
-    return $self->vertex_weight(
-        $self->heaviest_vertices->[0]
-    );
-}  # }}}
-
-sub min_weight {  # {{{
-    my $self = shift;
-    return $self->vertex_weight(
-        $self->lightest_vertices->[0]
-    );
-}  # }}}
-
 1;
 
 __END__
@@ -306,99 +258,86 @@ Graph::Weighted - A weighted graph implementation
 
   $g = Graph::Weighted->new(
       data => {
-          a => { b => 1, c => 2, },  # A vertex with two edges.
-          b => { a => 1, c => 3, },  # "
-          c => { a => 2, b => 3, },  # "
+          a => { b => 1, c => 2, },  # Nodes with two edges.
+          b => { a => 1, c => 3, },
+          c => { a => 2, b => 3, },
           d => { c => 1, },          # A vertex with one edge.
           e => {},                   # A vertex with no edges.
      }
   );
 
-  $g = Graph::Weighted->new(data => $object);  # See documentation.
+  $g->reset_graph;
 
   $g = Graph::Weighted->new();
-  $g->load(
-      [ [ 0, 1, 2, 0, 0, ],    # A vertex with two edges.
-        [ 1, 0, 3, 0, 0, ],    # "
-        [ 2, 3, 0, 0, 0, ],    # "
-        [ 0, 0, 1, 0, 0, ],    # A vertex with one edge.
-        [ 0, 0, 0, 0, 0, ], ]  # A vertex with no edges.
-  );
 
-  $data = $g->data;
+  $g->load(
+      [ [ 0, 1, 2 ],
+        [ 1, 0, 3 ],
+        [ 2, 3, 0 ], ]
+  );
 
   $w = $g->graph_weight;
 
-  $w = $g->vertex_weight($p);
-  $w = $g->vertex_weight($p, $w + 1);
+  $heaviest = $g->heaviest_vertices;
+  $lightest = $g->lightest_vertices;
 
-  $w = $g->edge_weight($p, $q);
-  $w = $g->edge_weight($p, $q, $w + 1);
+  $x = $g->vertex_weight($heaviest->[$i]) if @$heaviest;
+  $y = $g->vertex_weight($lightest->[$j]) if @$lightest;
 
-  $heavies = $g->heaviest_vertices;
-  $lights  = $g->lightest_vertices;
+  $x = $g->vertex_weight(0);
+  $y = $g->vertex_weight(0, $x + 1);
 
-  $x = $g->max_weight;  # Equal to the weight of the $heavies.
-  $y = $g->min_weight;  # Equal to the weight of the $lights.
+  $m = $g->matrix;
 
-  # Call any methods of the inherited Graph module.
-  @v = $g->vertices;
-  @v = $g->successors($p);  # etc.
-  $z = $g->MST_Kruskal;
-  $z = $g->APSP_Floyd_Warshall;
-  $z = $g->MST_Prim($p);
+=head1 ABSTRACT
+
+A weighted graph implementation
 
 =head1 DESCRIPTION
 
-A C<Graph::Weighted> object represents a subclass of 
-C<Graph::Directed> with weighted attributes that are taken from a 
-two dimensional matrix (HoH or NxN LoL) of numerical values.
-
-This module can also load the matrix portions of C<Math::Matrix>, 
-C<Math::MatrixReal>, and C<Math::MatrixBool> objects.
+A C<Graph::Weighted> object represents a subclass of C<Graph> with 
+weighted attributes that are taken from a 2D matrix (HoH or NxN LoL) 
+of numerical values.
 
 Initially, the weights of the vertices are set to the sum of their 
-outgoing edge weights.  This is mutable, however, and can be reset to 
+outgoing edge weights.  This is mutable, however, and can be set to 
 any value desired, after initialization, with the C<vertex_weight> 
-and C<edge_weight> methods.
+method.
 
 =head1 PUBLIC METHODS
 
 =over 4
 
-=item new %ARGUMENTS
+=item new HASH
 
 =over 4
 
-=item debug => 0 | 1
+=item debug 0 | 1
 
 Flag to invoke verbose mode while processing.  Defaults to zero.
 
-=item zero_edges => 0 | 1
+=item zero_edges 0 | 1
 
 Flag to add edges between vertices with a weight of zero.  Defaults to 
 zero.
 
-=item data => $HASHREF | $ARRAYREF | $OBJECT
+=item data HASHREF | ARRAYREF
 
-Two dimensional hash, (NxN) array, or known object reference to use 
-for vertices and weighted edges.
-
-The known objects are C<Math::Matrix>, C<Math::MatrixReal>, and 
-C<Math::MatrixBool>.
+Two dimensional hash or (2D, square) array reference to use for 
+vertices and weighted edges.
 
 =back
 
-=item load $HASHREF | $ARRAYREF | $OBJECT
+=item reset_graph
 
-Turn the given two dimensional hash, (NxN) array, or known object 
-reference into the vertices and weighted edges of a 
-C<Graph::Directed> object.
+Erase the graph's vertices, edges and attributes.
 
-The known objects are C<Math::Matrix>, C<Math::MatrixReal>, and 
-C<Math::MatrixBool>.
+=item load HASHREF | ARRAYREF
 
-=item data
+Turn the given two dimensional hash or (2D, square) array reference 
+into the vertices and weighted edges of a C<Graph> object.
+
+=item matrix
 
 Return the two dimensional hash used for vertices and weighted edges.
 
@@ -406,41 +345,32 @@ Return the two dimensional hash used for vertices and weighted edges.
 
 Get the total weight of the graph, by summing all the vertex weights.
 
-=item vertex_weight $VERTEX [, $WEIGHT]
+=item vertex_weight SCALAR [, SCALAR]
 
-Return the weight of a vertex.
+Return the weight of a vertex.  This method can also be used to set 
+the vertex weight, if a second argument is provided.
 
-(The vertices are just the keys of the data, not some glorified 
-object, by the way.)
+When the second argument is provided, the weight it represents is 
+distributed evenly to the vertex's outgoing edges, and the total 
+weight of the entire graph is adjusted accordingly.
 
-If a second argument is provided, the vertex weight is set to that 
-value and is distributed evenly to the vertex's outgoing edges, and 
-the total weight of the graph is adjusted accordingly.
+=item edge_weight SCALAR, SCALAR [, SCALAR]
 
-=item edge_weight $VERTEX, $SUCCESSOR [, $WEIGHT]
+Return the weight of an edge.  This method can also be used to set
+the edge weight, if a third argument is provided.
 
-Return the weight of an edge between the two given vertices.
-
-If a third argument is provided, the weight it represents is used
+When the third argument is provided, the weight it represents is used
 to replace the weight of the edge between the vertex (first argument)
-and it's successor (second argument).  Finally, the weight of the 
-vertex and the total weight of the graph are adjusted accordingly.
+and it's neighbor (second argument).  Lastly, the total weight of the 
+entire graph and the weight of the vertex are adjusted accordingly.
 
 =item heaviest_vertices
 
-Return an array reference of vertices with the most weight.
+Return the array reference of vertices with the most weight.
 
 =item lightest_vertices
 
-Return an array reference of vertices with the least weight.
-
-=item max_weight
-
-Return the weight of the heaviest vertices.
-
-=item min_weight
-
-Return the weight of the lightest vertices.
+Return the array reference of vertices with the least weight.
 
 =back
 
@@ -448,7 +378,7 @@ Return the weight of the lightest vertices.
 
 =over 4
 
-=item _debug @STUFF
+=item _debug ARRAY
 
 Print the contents of the argument array with a newline appended.
 
@@ -456,7 +386,7 @@ Print the contents of the argument array with a newline appended.
 
 =head1 SEE ALSO
 
-L<Graph::Base>
+L<Graph>
 
 =head1 TO DO
 
@@ -464,7 +394,7 @@ Handle clusters of vertices and sub-graphs.
 
 =head1 AUTHOR
 
-Gene Boggs E<lt>gene@cpan.orgE<gt>
+Gene Boggs E<lt>cpan@ology.netE<gt>
 
 =head1 COPYRIGHT AND LICENSE
 
